#!/perl -w
# NAME: vcproj04.pl
# AIM: To scan a VCPROJ file, and show the results
# This uses the services in fgscanvc.pl, to standardise the processing of a VCPROJ file
# so this is very different to vcproj03.pl, which had its own services to do the scan.
# 2009-06-05 also try to attempt to output what the project will create... exe,lib,dll,...
# 05/12/2008 geoff mclane http://geoffair.net/mperl
use strict;
use warnings;
unshift(@INC, 'C:/GTools/perl');
require 'fgutils.pl' or die "Unable to load fgutils.pl ...\n";
require 'fgdsphdrs.pl' or die "Unable to load fgdsphdrs.pl ...\n";
require 'fgscanvc.pl' or die "Unable to load fgscanvc.pl ...\n";
# log file stuff
my ($LF);
my $pgmname = $0;
if ($pgmname =~ /\w{1}:\\.*/) {
    my @tmpsp = split(/\\/,$pgmname);
    $pgmname = $tmpsp[-1];
}
my $outfile = "temp.$pgmname.txt";
open_log($outfile);

my $dbg_val = 4+2;    # 1=split defines, 2=no show defines, etc, 4=show sources;
my @warnings = ();
my $write_dsp = 1;
my $in_file = 'C:\Projects\freetype-2.3.9\builds\win32\vc2008\freetype.vcproj';
#my $in_file = 'C:\FG\27\TaxiDraw\msvc\7.1\TaxiDraw.vcproj';
#my $in_file = 'C:\FG\27\zlib-1.2.3\projects\visualc6\zlib.vcproj';
#my $in_file = 'C:\FG\27\FlightGear\projects\vc7.1\terrasync.vcproj';
#my $in_file = 'C:\FG\FGRUN\fgrunplib\fgrun.vcproj';
my $out_dsp = 'tempdsp.dsp';

my $help = <<EOF;
$pgmname [OPTIONS] in_file
OPTIONS:
 -? or -h        - This brief help.
 -dsp[=dsp_name] - Write DSP file (def=$out_dsp).
 -in=in_file     - Alternative to set input file.
EOF

parse_args(@ARGV);

prt( "$pgmname ... Scanning $in_file ...\n" );

# dbg_show_entering_files();
# dbg_show_source_files();
dbg_show_output_files(); # { $dbg_v21 = 1; $dbg_v24 = 1; }

my %h = process_VCPROJ( $in_file );

show_hash_results( $dbg_val, \%h );

write_hash_to_DSP( $out_dsp, \%h, 0 ) if ($write_dsp);

show_warnings();

close_log($outfile,1);
unlink($outfile);   # delete output file
exit(0);

##########################################################

sub prtw {
    my ($tx) = shift;
    $tx =~ s/\n$//;
    prt("$tx\n");
    push(@warnings,$tx);
}

sub show_warnings {
    if (@warnings) {
        prt( "\nGot ".scalar @warnings." WARNINGS ...\n" );
        foreach my $line (@warnings) {
            prt("$line\n" );
        }
        prt("\n");
    } else {
        prt("\nNo warnings issued.\n\n");
    }
}

sub give_help {
    prt( $help );
	mydie("In file must exist ...\n");
}

sub chk_arg {
    my ($arg, @av) = @_;
    fatal( "Invalid $arg - needs value ... -? for help ... aborting!\n" ) if !(@av);
}

sub parse_args { # @ARGV
	my (@av) = @_;
    my ($dn, @tv);
	while (@av) {
		my $arg = $av[0];
		if (substr($arg,0,1) eq '-') {
			if (($arg eq '-?')||($arg eq '-h')) {
				give_help();
            } elsif ($arg =~ /^-in=(.+)/) {
    			$in_file = $1;
	    		prt( "Set in file to $in_file ...\n" );
			} else {
				mydie( "ERROR: Unknown option [$arg] ... try -? ... aborting!\n" );
			}
		} else {
			# bare item - assume INPUT file
			$in_file = $arg;
			prt( "Set in file to $in_file ...\n" );
		}
		shift @av;
	}
}

# eof - vcproj04.pl

